% % this script computes the marginal posterior density of the coefficient 
% % assoicated with the endogenous variable (gam)
% % outputs: 
% % pgamhat: marginal posterior density of gam
% % Egam: posterior mean of gam
% % gamstd: posterior std dev of gam
% %
% % See:
% % Chan, J.C.C. and Tobias, J.L. (2015). "Priors and Posterior
% % Computation in Linear Endogenous Variables Models with Imperfect
% % Instruments," Journal of Applied Econometrics, 30(4), 650-674
% %
% % =======================================================================
   
M = size(store_theta,1);
ngrid = 500;
gamgrid = linspace(-.5,.5,ngrid)';  % grid for gam
store_pgam = zeros(ngrid,1);
tic;
for j=1:M   
    a = store_a(j,:)';        
    sigv = sqrt(a(1));
    beta0 = a(2);
    beta1 = a(3);
    beta2 = a(4);
    a0 = a(5);
    a1 = a(6);
    a2 = a(7);
    b = a(8);
    c = sqrt(a(9));    
    bmg = b-gamgrid;
    sigu2 = c^2+bmg.^2*sigv^2;
    alp1 = a1+bmg*beta1;
    alp2 = a2+bmg*beta2;
    rho2 = bmg.^2*sigv^2./sigu2;
        % ignore the priors for some elements in theta as they are not important
    tempalp0 = -.5*log(2*pi) + .5*log(invVtheta(1,1)) -.5*invVtheta(1,1)*(a0+bmg*beta0).^2;
    tempalp1 = -.5*log(2*pi) + .5*log(invVtheta(2,2)) -.5*invVtheta(2,2)*alp1.^2;
    tempalp2 = -.5*log(2*pi) + .5*log(invVtheta(3,3)) -.5*invVtheta(3,3)*alp2.^2;
    tempJ = log(sigv) - .5*log(sigu2);
    detSig = (1-rho2).*sigu2*sigv^2;
    tempSig = -.5*(nu0+3)*log(detSig) -.5./detSig.*(sigv^2 + sigu2);
    if priorflag == 1
        pgam = exp(tempalp0 + tempalp1 + tempalp2 + tempSig + tempJ);        
    elseif priorflag == 2
        ind1 = abs(alp1)<=abs(gamub*beta1);
        ind2 = abs(alp2)<=abs(gamub*beta2);       
        pgam = exp(tempalp0 + tempalp1 + tempalp2 + tempSig + tempJ).*ind1.*ind2;        
    end
    store_pgam = pgam + store_pgam;
end
pgamhat = store_pgam/M;
kgam = (gamgrid(3)-gamgrid(2))*sum(pgamhat);
pgamhat = pgamhat/kgam;
Egam = gamgrid'*pgamhat/sum(pgamhat);
Egam2 = (gamgrid.^2)'*pgamhat/sum(pgamhat);
gamstd = sqrt(Egam2-Egam^2);
figure;
plot(gamgrid,pgamhat); title('\gamma'); box off;
set(gcf,'Position',[100 100 500 400]);
        