% % this script computes the marginal posterior density of the error 
% % correlation (rhouv)
% % outputs: 
% % prhouvhat: marginal posterior density of rhouv
% % Erhouv: posterior mean of rhouv
% % rhouvstd: posterior std dev of rhouv
% %
% % See:
% % Chan, J.C.C. and Tobias, J.L. (2015). "Priors and Posterior
% % Computation in Linear Endogenous Variables Models with Imperfect
% % Instruments," Journal of Applied Econometrics, 30(4), 650-674
% %
% % =======================================================================
M = size(store_theta,1);
ngrid = 500;
rhouvgrid = linspace(-.98,.98,ngrid)';
store_prhouv = zeros(ngrid,1);
for j=1:M 
    theta = store_theta(j,:)';
    Sig = store_Sig(j,:)';
    tempsig = Sig(2)/Sig(3);
    a = [Sig(3) theta(na+2) theta(end-1) theta(end)...
            theta(1)-tempsig*theta(na+2) theta(2)-tempsig*theta(end-1)...
            theta(3)-tempsig*theta(end) theta(na+1)+tempsig Sig(1)-Sig(2)^2/Sig(3)];        
    sigv = sqrt(a(1));
    beta0 = a(2);
    beta1 = a(3);
    beta2 = a(4);
    a0 = a(5);
    a1 = a(6);
    a2 = a(7);
    b = a(8);
    c = sqrt(a(9));  
    betai = theta(na+1+2:na+1+7);
    ai = theta(8:13)-tempsig*betai;
    rho2 = rhouvgrid.^2;
    sigu2 = c^2./(1-rho2); sigu = sqrt(sigu2);
    temp1 = rhouvgrid.*sigu./sigv;
    alp0 = a0 + temp1*beta0;
    alp1 = a1 + temp1*beta1;
    alp2 = a2 + temp1*beta2;
    gam = b - temp1;    
        % ignore the priors for some elements in theta as they are not important
    tempalp0 = -.5*invVtheta(1,1)*alp0.^2;
    tempalp1 = -.5*invVtheta(2,2)*alp1.^2;
    tempalp2 = -.5*invVtheta(3,3)*alp2.^2;
    tempalpi = 0;
    for ii = 1:6
        alpi = ai(ii)+temp1*betai(ii);
        tempalpi = tempalpi -.5*invVtheta(7+ii,7+ii)*alpi.^2;
    end   
    tempJ = -log(1-rho2);    
    detSig = (1-rho2).*sigu2*sigv^2;
    tempSig = -.5*(nu0+3)*log(detSig) -.5./detSig.*(sigv^2 + sigu2);
    if priorflag == 1; 
        prhouv = exp(tempalp0 + tempalp1 + tempalp2 + tempalpi + tempSig + tempJ);        
    else        
        ind1 = abs(alp1)<=abs(gamub*beta1);
        ind2 = abs(alp2)<=abs(gamub*beta2);       
        prhouv = exp(tempalp0 + tempalp1 + tempalp2 + tempalpi + tempSig + tempJ).*ind1.*ind2;
    end
    store_prhouv = prhouv + store_prhouv;
end
prhouvhat = store_prhouv/M;
krhouv = (rhouvgrid(3)-rhouvgrid(2))*sum(prhouvhat);
prhouvhat = prhouvhat/krhouv;
Erhouv = rhouvgrid'*prhouvhat/sum(prhouvhat);
Erhouv2 = (rhouvgrid.^2)'*prhouvhat/sum(prhouvhat);
rhouvstd = sqrt(Erhouv2-Erhouv^2);

figure; plot(rhouvgrid,prhouvhat); title('\rho_{uv}'); box off;
set(gcf,'Position',[100 100 500 400]);


        