% % this script computes the marginal posterior density of the error 
% % variance of the outcome equation (sigu2)
% % outputs: 
% % psigu2hat: marginal posterior density of sigu2
% % Esigu2: posterior mean of sigu2
% % sigu2std: posterior std dev of sigu2
% %
% % See:
% % Chan, J.C.C. and Tobias, J.L. (2015). "Priors and Posterior
% % Computation in Linear Endogenous Variables Models with Imperfect
% % Instruments," Journal of Applied Econometrics, 30(4), 650-674
% %
% % =======================================================================
M = size(store_theta,1);
ngrid = 500;
sigu2grid = linspace(.01,.3,ngrid)';
store_psigu2 = zeros(ngrid,1);
for j=1:M 
    theta = store_theta(j,:)';
    Sig = store_Sig(j,:)';
    tempsig = Sig(2)/Sig(3);
    a = [Sig(3) theta(na+2) theta(end-1) theta(end)...
            theta(1)-tempsig*theta(na+2) theta(2)-tempsig*theta(end-1)...
            theta(3)-tempsig*theta(end) theta(na+1)+tempsig Sig(1)-Sig(2)^2/Sig(3)];
        
    sigv = sqrt(a(1));
    beta0 = a(2);
    beta1 = a(3);
    beta2 = a(4);
    a0 = a(5);
    a1 = a(6);
    a2 = a(7);
    b = a(8);
    c = sqrt(a(9));  
    temp1 = sqrt((sigu2grid-c^2)/sigv^2);
    id = find(imag(temp1)>0,1,'last');
    rho2  = 1-c^2./sigu2grid;
    alp0a = a0 + temp1*beta0;
    alp0b = a0 - temp1*beta0;    
    alp1a = a1 + temp1*beta1;
    alp1b = a1 - temp1*beta1;    
    alp2a = a2 + temp1*beta2;
    alp2b = a2 - temp1*beta2;
    gama = b - temp1;
    gamb = b + temp1;    
        % ignore the priors for some elements in theta as they are not important
    tempalp0a = -.5*invVtheta(1,1)*alp0a.^2;
    tempalp0b = -.5*invVtheta(1,1)*alp0b.^2;
    tempalp1a = -.5*invVtheta(2,2)*alp1a.^2;
    tempalp1b = -.5*invVtheta(2,2)*alp1b.^2;
    tempalp2a = -.5*invVtheta(3,3)*alp2a.^2;
    tempalp2b = -.5*invVtheta(3,3)*alp2b.^2;    
    tempJ = -.5*log(sigu2grid.*(sigu2grid - c^2));    
    detSig = (1-rho2).*sigu2grid*sigv^2;
    tempSig = -.5*(nu0+3)*log(detSig) -.5./detSig.*(sigv^2 + sigu2grid);           
    if priorflag == 1; 
        psigu2a = exp(tempalp0a + tempalp1a + tempalp2a + tempSig + tempJ);
        psigu2b = exp(tempalp0b + tempalp1b + tempalp2b + tempSig + tempJ);        
    else        
        ind1a = abs(alp1a)<=abs(gamub*beta1);
        ind1b = abs(alp1b)<=abs(gamub*beta1);
        ind2a = abs(alp2a)<=abs(gamub*beta2);
        ind2b = abs(alp2b)<=abs(gamub*beta2);
        psigu2a = exp(tempalp0a + tempalp1a + tempalp2a + tempSig + tempJ).*ind1a.*ind2a;
        psigu2b = exp(tempalp0b + tempalp1b + tempalp2b + tempSig + tempJ).*ind1b.*ind2b;
    end
    psigu2a(1:id) = 0;  psigu2b(1:id) = 0;
    ksigu2a = (sigu2grid(3)-sigu2grid(2))*sum(psigu2a);
    ksigu2b = (sigu2grid(3)-sigu2grid(2))*sum(psigu2a);
    if sum(psigu2a) ~= 0 && sum(psigu2a) ~= 0
        psigu2 = (psigu2a/ksigu2a + psigu2b/ksigu2b)/2;
    else
        psigu2 = 0;
    end
    store_psigu2 = psigu2 + store_psigu2;    
end
psigu2hat = store_psigu2/M;
ksigu2 = (sigu2grid(3)-sigu2grid(2))*sum(psigu2hat);
psigu2hat = psigu2hat/ksigu2;
Esigu2 = sigu2grid'*psigu2hat/sum(psigu2hat);
Esigu22 = (sigu2grid.^2)'*psigu2hat/sum(psigu2hat);
sigu2std = sqrt(Esigu22-Esigu2^2);

figure; plot(sigu2grid,psigu2hat); title('\sigma^2_{u}'); box off;
set(gcf,'Position',[100 100 500 400]);
        